import { Component, Inject, Input, ApplicationRef, EventEmitter, SimpleChange, SimpleChanges, OnChanges, OnInit, InjectionToken, Injectable, ɵɵNgOnChangesFeature, ViewChild } from '@angular/core'
import { UserService } from '../user.service'
import { User } from '../user-list/userlist.model'
import { Subscription } from 'rxjs';
import { MatDialog, MatDialogRef, MAT_DIALOG_DATA} from '@angular/material/dialog';
import { MatSnackBar } from "@angular/material/snack-bar";
import { MatSort, Sort } from '@angular/material/sort'
import { MatTableDataSource } from '@angular/material/table';
import { PageEvent } from '@angular/material/paginator';
import { SelectionModel } from '@angular/cdk/collections'
import { element } from 'protractor';

export interface DialogData {
  feide_id: string,
  name: string,
  role: string,
  active: boolean,
  tags: [string]
}

interface EditData {
  feide_id: string,
  active: boolean,
  role: string,
  upload: boolean,
  tags: [string]
}

@Component({
    selector: 'app-user-list',
    templateUrl: './user-list.component.html',
    styleUrls: ['./user-list.component.scss']
})

@Injectable({providedIn: 'root'})
export class UserListComponent{
  displayedColumns: string[] = ['select', 'name', 'role', 'lastLogin', 'dateRegistered', 'active', 'options'];
  users: User[] = [];
  private usersSub: Subscription
  isLoadingResults = true
  editId: string
  numberOfUsers = 2
  usersPerPage = 25
  pageIndex = 0
  pageSizeOption = [1, 3, 5, 10, 25]
  constructor(public userService: UserService, public dialog: MatDialog) {}
  dataSource = new MatTableDataSource(this.users);
  selection = new SelectionModel(true, [])

  @ViewChild(MatSort) sort: MatSort;

  ngOnInit() {
    this.getUserList()

    this.userService.getUserCount()                                       
            .subscribe((userCount) => {
                this.numberOfUsers = userCount.count
      })

    this.usersSub = this.userService.getUserUpdateListener()
      .subscribe(() => {
          this.getUserList()
      })
  }

  ngOnDestroy() {
    this.usersSub.unsubscribe()
  }
  
  getUserList() {
    this.isLoadingResults = true
    this.userService.getUsers(this.usersPerPage, this.pageIndex)          
      .subscribe(userData => {
        this.users = userData.users
        this.isLoadingResults = false

        this.dataSource = new MatTableDataSource(this.users);

        this.dataSource.sortingDataAccessor = (item, property) => {
          if(!item[property]) {
              return this.sort.direction === "asc" ? '3' : '1'
          }
          return '2' + item[property].toLocaleLowerCase()
          }
        this.dataSource.sort = this.sort;
      })
  }

  editUser(user) {
    const dialogRef = this.dialog.open(UserEditDialog, {
      width: '450px',
      restoreFocus: false,
      data: user});
  }

  deleteUser(user) {
    const dialogRef = this.dialog.open(UserDeleteDialog, {
      width: '450px',
      restoreFocus: false,
      data: user});
  }

  onPageChange(pageData: PageEvent) {                                     //Runs when changing pages in the userlist
    this.usersPerPage = pageData.pageSize
    this.pageIndex = pageData.pageIndex
    this.getUserList()
  }

  masterToggle() {                                                        //Checks off all checkboxes, if all are selected it de-selects all
    this.isAllSelected() ?
        this.selection.clear() :
        this.dataSource.data.forEach(element => this.selection.select(element))
  }

  isAllSelected() {                                                       //checks if all checkboxes are selected
    const numSelected = this.selection.selected.length
    const numRows = this.dataSource.data.length
    return numSelected === numRows
  }
}

@Component({
  selector: 'user-edit-dialog',
  templateUrl: './user-edit-dialog.html',
  styleUrls: ['./user-edit-dialog.scss']
})
export class UserEditDialog {
  constructor(@Inject(MAT_DIALOG_DATA) public data: DialogData, public userService: UserService, private _snackBar: MatSnackBar) {}
  role: String
  active: boolean
  valueChanged: boolean
  originalData: EditData
  editedData: EditData
  upload:boolean = this.data.tags.includes('upload') ? true : false

  ngOnInit() {
    
    if (this.data.tags.includes('upload')) {
      this.data.tags.splice(this.data.tags.indexOf('upload'),1)
    }

    this.originalData = {
      feide_id: this.data.feide_id,
      active: this.data.active,
      role: this.data.role,
      upload: this.upload,
      tags: this.data.tags
    }

    this.editedData = JSON.parse(JSON.stringify(this.originalData)); // Copy original to editedData
  }

  onChange(e){                                                                        // monitor for object changes, controlling submit button on/off
    if(JSON.stringify(this.originalData) != JSON.stringify(this.editedData)) {
      this.valueChanged = true
    } else {
      this.valueChanged = false
    }
  }

  onEdit() {
    if(this.editedData.role == 'student' && this.editedData.upload) {                
      this.editedData.tags.push('upload')                                             // add upload tag to user object if true
      delete this.editedData["upload"]
    } else {
      delete this.editedData.tags['upload']                                           // remove if not
    }

    this.userService.editUser(this.editedData)
      .subscribe(resdata => {
        this.openSnackBar(`User ${this.data.name} edited`)
        this.userService.usersUpdated.next(true)                                      // Trigger update of userlist
      },
      errorMessage => {
        this.openSnackBar(`An error occured: ${errorMessage.error.message}`)
      })
  }
  
  
  openSnackBar(message: string) {
    this._snackBar.open(message, 'Dismiss', {
      duration:2000
    })
  }
}

@Component({
  selector: 'user-delete-dialog',
  templateUrl: './user-delete-dialog.html',
  styleUrls: ['./user-delete-dialog.scss']
})
export class UserDeleteDialog {
  constructor(@Inject(MAT_DIALOG_DATA) public data: DialogData, public userService: UserService, private _snackBar: MatSnackBar) {}

  onDelete(){
    this.userService.deleteUser(this.data.feide_id)
      .subscribe(resdata => {
        this.openSnackBar(`User ${this.data.name} deleted`)
        this.userService.usersUpdated.next(true)
      },
      errorMessage => {
        this.openSnackBar(`An error occured: ${errorMessage.error.message}`)
      })
  }

  openSnackBar(message: string) {
    this._snackBar.open(message, 'Dismiss', {
      duration:2000
    })
  }
}