import { Component, ViewChild, AfterViewInit } from "@angular/core";
import { File } from "./file-list.model";
import { FileService } from "../file.service";
import { Subscription } from "rxjs";
import {
  animate,
  state,
  style,
  transition,
  trigger,
} from "@angular/animations";
import { PageEvent } from "@angular/material/paginator";
import { MatSort, Sort } from "@angular/material/sort";
import { MatTableDataSource } from "@angular/material/table";
import { SelectionModel } from "@angular/cdk/collections";
import { ActivatedRoute } from "@angular/router";
import { MatSnackBar } from "@angular/material/snack-bar";
import { FileSaverService } from "ngx-filesaver";

@Component({
  selector: "app-file-list",
  templateUrl: "./file-list.component.html",
  styleUrls: ["./file-list.component.scss"],
  animations: [
    trigger("detailExpand", [
      state("collapsed, void", style({ height: "0px", minHeight: "0" })),
      state("expanded", style({ height: "*" })),
      transition(
        "expanded <=> collapsed, void => expanded",
        animate("225ms cubic-bezier(0.4, 0.0, 0.2, 1)")
      ),
    ]),
  ],
})
export class FileListComponent implements AfterViewInit {
  displayedColumns: string[] = [
    "select",
    "original_filename",
    "filetype",
    "size",
    "date_added",
    "expansion_status",
  ];
  files: File[] = [];
  isLoadingResults = true;
  expandedFile: File | null;
  numberOfFiles = 10;
  filesPerPage = 25;
  pageIndex = 0;
  pageSizeOption = [25, 50, 75, 100, 250];
  dataSource = new MatTableDataSource(this.files);
  selection = new SelectionModel(true, []);

  numSelected = this.selection.selected.length;

  private filesSub: Subscription;
  public action: string;

  constructor(
    public fileService: FileService,
    private route: ActivatedRoute,
    private _snackBar: MatSnackBar,
    private FileSaverService: FileSaverService
  ) {}

  @ViewChild(MatSort) sort: MatSort;

  ngOnInit() {
    // get the number of files from the database for the paginator
    this.fileService.getFileCount().subscribe((fileCount) => {
      this.numberOfFiles = fileCount.count;
    });

    // get the files
    this.getFileList();

    // subscribe for changes, and pull new files if there are any
    this.filesSub = this.fileService.getFilesUpdateListner().subscribe(() => {
      this.getFileList();
      console.log("New file(s) uploaded");
    });
  }

  // initiate sorting after loading the view
  ngAfterViewInit() {
    this.dataSource.sort = this.sort;
  }

  // remember do unsubscribe when the view is getting destroyed
  ngOnDestroy() {
    this.filesSub.unsubscribe();
  }

  // get the file list
  getFileList() {
    this.isLoadingResults = true;
    this.fileService
      .getPageFiles(this.filesPerPage, this.pageIndex)
      .subscribe((fileList) => {
        // initiate sorting
        this.dataSource = new MatTableDataSource(fileList);
        this.dataSource.sortingDataAccessor = (item, property) => {
          if (!item[property]) {
            return this.sort.direction === "asc" ? "3" : "1";
          }
          return "2" + item[property].toLocaleLowerCase();
        };

        this.dataSource.sort = this.sort;
        this.isLoadingResults = false;
      });
  }

  onPageChange(pageData: PageEvent) {
    //Runs when changing pages in the filelist
    this.filesPerPage = pageData.pageSize;
    this.pageIndex = pageData.pageIndex;
    this.getFileList();
  }

  // code for checkbox-toggle
  masterToggle() {
    this.isAllSelected()
      ? this.selection.clear()
      : this.dataSource.data.forEach((element) =>
          this.selection.select(element)
        );
  }

  // func if all checkboxes are checked
  isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.dataSource.data.length;
    return numSelected === numRows;
  }

  // what action to do for checkboxes
  selectedAction() {
    switch (this.action) {
      case "reanalyze":
        console.log("Reanalyzing selected files");
        //Creates an array only containing the sha256's
        var data = this.selection.selected.map((x) => x.sha256); 
        var source = "virustotal";
        this.fileService.reanalyze(data, source).subscribe((data) => {
          console.log(data);
        });
        break;

      case "download":
        console.log("Downloading selected files");
        //Creates an array only containing the sha256's
        var data = this.selection.selected.map((x) => x.sha256); 
        this.fileService.downloadFiles(data).subscribe(
          (response) => {
            this.openSnackBar(`Downloading ${data.length} files`);
            this.FileSaverService.save(
              response,
              `Malware-repo_${Date.now()}.zip`
            );
          },
          (error) => {
            console.log(error);
            this.openSnackBar(`An error occured.`);
          }
        );
        break;

      case "delete":
        console.log("Deleting selected files");
        //Creates an array only containing the sha256's
        var data = this.selection.selected.map((x) => x.sha256); 
        break;

      case "favorite":
        console.log("Favoriting selected files");
        //Creates an array only containing the sha256's
        var data = this.selection.selected.map((x) => x.sha256); 
        break;

      default:
        console.log("No action selected");
        break;
    }
  }

  openSnackBar(message: string) {
    this._snackBar.open(message, "Dismiss", {
      duration: 2000,
    });
  }
}
