const express = require("express");
const router = express.Router();
const fileService = require("../../db/file-service");
const multer = require("../../middleware/multer");
const FileClass = require("../../pipeline/fileclass");
const virustotal = require("../../pipeline/virustotal/virustotal");
const routeService = require("../../db/route-service");
const download = require("../../models/download");
const {
	authorize,
} = require("../../auth/passport-openid-connect/lib/authorize");
const {filefilter} = require('../../auth/filefilter')

module.exports = function (app, express, passport) {
	/**
	 * @desc Get all file db entries
	 * @route /api/v1/file
	 * @res returns 201 if it successfully returns files
	 */
	router.get("", authorize({ all: true }), (req, res, next) => {
		const size = req?.query?.size || 100;
		const index = req?.query?.index || 0;
		const search = req?.query?.text;

		fileService
			.getFiles(size, index, search)
      .then(async (fileList) => {
        //removes files the user dont have access to
				let filtered = await filefilter(fileList, req.user)
				res.send(filtered);
			})
			.catch((err) => {
				console.error(err);
				res.sendStatus(500);
			});
	});

	/**
	 * @desc Upload a file (the file itself)
	 * @route /api/v1/file/upload - No params, but file(s) as object
	 * @res 400 if post is misformed, 200 if file is uploaded
	 */
	router.post(
    "/upload",
    //only admin, reasearchers and students with upload tag may upload
		authorize({ role: ["admin", "researcher"], tags: ["upload"] }),
		multer,
		async function (req, res, next) {
      if (req.files) {
        //make new upload object
				new FileClass(
					process.env.tempfilePath,
					req.files.map((x) => x.originalname),
					{ uploader: req?.user?.data?.name },
          async function (id) {
            //send id of the upload object
						res.status(201).json({
							id: id,
						});
					}
				);
			} else {
				res.sendStatus(400);
			}
		}
	);

	/**
	 * @desc Counts number of files in DB
	 * @route /api/v1/file/count
	 * @res returns 201 if it successfully returns files
	 */
	router.get("/count", authorize({ all: true }), (req, res, next) => {
		fileService
			.getFileCount()
			.then((fileCount) => {
				res.status(201).json({
					count: fileCount,
				});
			})
			.catch((err) => {
				//WINSTON ERROR
				res.sendStatus(500);
			});
	});

	/**
	 * @desc reanalyze file(s)
	 * @route /api/v1/file/reanalyze -
	 * @param hash of file
	 * @res 40 if no hash sent, 200 if added to queue
	 */
	router.post(
		"/reanalyze",
		authorize({ role: ["admin", "researcher"] }),
		(req, res, next) => {
			if (req?.body?.hash) {
				if (req.body.analyze_source == "virustotal") {
					virustotal.addToQueue(req.body.hash);
				} else if (req.analyze_source == "static") {
					routeService.reanalyze(req.body.hash);
				} else {
					routeService.reanalyze(req.body.hash);
					virustotal.addToQueue(req.body.hash);
				}
				res.status(200).json({
					message: "file added to queue",
				});
			} else {
				res.status(400).json({
					message: "no hash sent",
				});
			}
		}
	);

	/**
	 * @desc Get a specific file (the file itself)
	 * @route /api/v1/file/download - Parameter for file-hash
	 * @res 404 if file not found, 200 if files found
	 */
	router.post(
    "/download",
    //all logged in users can download
		authorize({ all: true }),
		async function (req, res, next) {
			let password = req?.body?.password || "infected";

      if (req?.body?.files) {
				let downloadObject = {
					downloader: req?.user?.data?.name,
					password: password,
					files: req.body.files,
				};
				//create a new db entry for this download.
				new download(downloadObject).save().catch((err) => {
					console.error(err, req?.body?.files);
        });
        //add attachment
				res.attachment(`Malware-repo_${Date.now()}`);
				await routeService.downloadFiles(req.body.files, password, res);
			} else {
				res.sendStatus(400);
			}
		}
	);

	/**
	 * @desc update filetags
	 * @route /api/v1/file/tags
	 * @res 200 if tags updated, 400 if no tags
	 */
	router.put(
		"/tags",
		authorize({ role: ["admin", "researcher"] }),
    (req, res, next) => {
      //if request contains both tags and hash
      if (req?.body?.hash && req?.body?.tags) {
        //update the file with the new tags
				fileService
					.tags(req.body.hash, req.body.tags)
					.then((data) => {
						console.log(data);
						res.status(200).json({
							message: "Successfully updated tags",
						});
					})
					.catch(() => {
						res.sendStatus(500);
					});
			} else {
				res.sendStatus(400);
			}
		}
	);

	//ALL WILDCARDS BELOW THIS LINE!
	/**
	 * @desc Get a specific file (data-object)
	 * @route /api/v1/file/:sha256 - Parameter for file-hash
	 * @res 404 if file not found, 200 if file found
	 */
	router.get("/:hash", authorize({ all: true }), function (req, res, next) {
		const hash = req?.params?.hash; // Save hash from params to var
		if (hash.length == 64) {
			fileService
				.findFile(hash) // Check if hash exists in db
				.then((file) => {
					if (!file) {
						// If file not exists
						return res.sendStatus(404);
					}
					res.send(file); // File exists, send file-object
				});
		} else {
			res.sendStatus(404);
		}
	});

	/**
	 * @desc Get a specific file (the file itself)
	 * @route /api/v1/file/:sha256/file - Parameter for file-hash
	 * @res 404 if file not found, 200 if file found
	 */
	router.get(
		"/:hash/file",
		authorize({ all: true }),
		async function (req, res, next) {
			const hash = req.params.hash; // Save hash from params to var
			let password = req?.query?.password || "infected";

			if (hash.length == 64) {
				fileService
					.findFile(hash) // Check if hash exists in db
					.then((file) => {
						if (!file) {
							// If file dont exists
							return res.sendStatus(404);
						}

						let downloadObject = {
							downloader: req?.user?.data?.name,
							password: password,
							files: req.body.files,
						};
						//create a new db entry for this download.
						new download(downloadObject).save().catch((err) => {
							console.error(err, req?.body?.files);
						});

						res.attachment(`${hash}.zip`);
						routeService
							.downloadFiles(hash, password, res)
							.then(() => {})
							.catch((err) => {
								console.error(err);
								return res.sendStatus(500);
							});
					});
			} else {
				res.sendStatus(404);
			}
		}
	);
	return router;
};
