const filetype = require('file-type')
const fsPromises = require('fs').promises
const fileService = require('../db/file-service')
const path = require('path')
const pipelineService = require('../db/pipeline-service')
const hasha = require('hasha')
const math = require('math')
const uploadModel = require('../models/uploadStatus')
const MAXFILES = Number(process.env.filesinfileclass)

//this class will handle all files from the upload until they are stored in the database
class fileClass {
  constructor(folder, filenames, options = {}, cb = null) {
    this.splits = []
    this.eventEmitter = fileService.uploadStatus()
    this.docker = 'unzip'
    this.unzippedCount = 0
    this.currentDir = process.env.tempfilePath
    console.log('filenames: ', filenames)

    if ('reanalyze' in options) {
      this.reanalyze = true
    }

    if ('uploader' in options) {
      this.uploader = options.uploader
    }

    if ('id' in options) {
      this.id = options.id
      this.currentPath = this.currentDir + this.id
      
      console.log('ID: ', this.id)
      this.main(folder, filenames)
    } else {
      this.mkTmpDir(process.env.tempfilePath).then(directory => {
        this.id = path.basename(directory)
        this.currentPath = this.currentDir + this.id
        
        console.log('ID: ', this.id)
        if (cb) {
          cb(this.id)
        }
        this.main(folder, filenames)
      })
    }
  }

  //add the specified files to this fileclass
  async addFiles(folder, filenames) {
    if (Array.isArray(filenames)) {
      await Promise.all(
        filenames.map(async (file) => {
          await this.addToFolder(`${folder}/${file}`)
        })
      )
    } else {
      await this.addToFolder(`${folder}/${filenames}`)
    }
    return
  }

  //remove the whole fileclass object
  remove() {
    console.log('removing', this.id)
    fsPromises.rmdir(this.currentPath).catch((err) => {
      console.log('cant remove folder. Most likely files in it. ID: ', this.id, err)
    })
  }

  //list all files in the object folder
  async listFiles() {
    console.log('Current path: ', this.currentPath)
    return fsPromises.readdir(this.currentPath)
  }

  //are there any compressed files in the object folder
  async containsCompressed() {
    let containsCompressed = false

    //currently supported compression types
    const compressed = [
      'application/zip',
      'application/x-rar',
      'application/x-tar',
      'application/gzip',
    ]

    let files = await this.listFiles()
    console.log('list of files: ', files)
    await Promise.all(
      files.map(async (file) => {
        console.log(`${this.currentPath}/${file}`)
        let type = await filetype.fromFile(`${this.currentPath}/${file}`)

        //filetype returns 'undefined' when the type is not known
        if (type) {
          console.log('Filetype: ', type.mime)
          if (compressed.includes(type.mime)) {
            await this.renameFiles(file)
            containsCompressed = true
          }
        }
      })
    ).catch((err) => {
      console.error(err)
    })

    if (containsCompressed) {
      this.updateUploadStatus('contains_compressed', true)
      console.log('file is zipped')
      return true
    } else {
      console.log('file is not zipped')
      return false
    }
  }

  //add the specified file to the object folder
  addToFolder(filepath) {
    return fsPromises.rename(filepath,
      `${this.currentPath}/${path.basename(filepath)}`
    )
  }

  //moves the object folder the next part of the analysis
  async moveTo(folder) {
    console.log('moving to:', folder)
    await fsPromises.rename(this.currentPath, `${folder}/${this.id}`, {})
    this.currentDir = folder
    this.currentPath = this.currentDir + this.id
    console.log('moved to: ', this.currentPath)
  }

  //add the files in the folder object to database
  async addDbEntry() {
    console.log('into addDBEntry', this.id)
    let files = await this.listFiles()

    await Promise.all(
      files.map(async (file) => {
        let currentFile = `${this.currentPath}/${file}`

        let file_md5 = await hasha.fromFile(currentFile, { algorithm: 'md5' })
        let file_sha256 = await hasha.fromFile(currentFile, {
          algorithm: 'sha256',
        })

        console.log(path.resolve(currentFile))
        let filesize = (await fsPromises.stat(path.resolve(currentFile)).catch(err => {
          console.error(err, currentfile)
        })).size
        console.log("Filesize:", filesize)

        
        console.log('adding new file')
        let newfile = {
          sha256: file_sha256,
          md5: file_md5,
          original_filename: file,
          size: filesize,
          uploaded_by: this.uploader,
          analyzed_info: {}
        }

        await fileService.createNew(newfile).then((file) => {
          console.log('file added: ', file.original_filename)
          this.updateUploadStatus('added_database', true)
        }).catch(err => {
          console.error(err, file)
        })
        
      })
    )
    console.log('done in DBentry', this.id)
    return
  }

  //delete the file if it already exists in the database
  async deleteExisting() {
    console.log('into delete exisiting', this.id)
    let files = await this.listFiles()

    await Promise.all(
      files.map(async (file) => {
        let file_sha256 = await hasha.fromFile(`${this.currentPath}/${file}`, {
          algorithm: 'sha256',
        })

        let dbfile = await fileService.findFile(file_sha256)
        if (dbfile) {
          console.log('deleting existing file: ', file)
          await fsPromises.unlink(`${this.currentPath}/${file}`)
        }
      })
    )
    console.log('done deleting', this.id)
    return
  }

  //responsible for unzipping the compressed files in the folder object
  async unzip() {
    console.log('started unzip', this.id)
    if (await this.containsCompressed()) {
      if (this.unzippedCount < 10) {
        this.unzippedCount += 1

        if (this.currentDir != process.env.dockerFilePath) {
          this.docker = 'unzip'
          await this.moveTo(process.env.dockerFilePath)
        }

        console.log('starting docker unzip')
        await pipelineService.docker_unzip(this)
        await this.split()
        await this.unzip()
      } else {
        console.log('unzipped too many times')
        await this.moveTo(process.env.unzippableFilePath)
      }
    } else {
      this.updateUploadStatus('unzipped', true)
      this.updateUploadStatus('contains_compressed', false)
      console.log('does not contain compressed', this.id)
      return
    }
  }

  //add the folder object to the analysis queue
  async analyze() {
    console.log('into analyze', this.id)
    this.docker = 'analyze'
    await this.moveTo(process.env.dockerFilePath)
    await pipelineService.addAllSources(this)
    return
  }

  //splits the folder object into several objects if there are too many files
  async split() {
    let that = this
    return new Promise(async (resolve, reject) => {
      console.log('started splitting', that.id)
      let files = await that.listFiles()
      let count = files.length

      if (count > MAXFILES) {

        //splits the files into arrays where max number of files in an array is MAXFILES
        let fileArray = new Array(Math.ceil(count / MAXFILES)).fill().map(_ => files.splice(0, MAXFILES))
        fileArray.splice(0,MAXFILES) // do this to keep the first number of files in this fileclass less than MAXFILES
        console.log('FileArray: ', fileArray)

        await Promise.all(
          fileArray.map(async (fileList) => {
            await this.mkTmpDir(process.env.tempfilePath).then(async directory => {
              let id = path.basename(directory)
              await Promise.all(
                fileList.map(async (file) => {
                  await fsPromises.rename(`${that.currentPath}/${file}`, `${directory}/${file}`)
                }))
                new fileClass(null, null, { 'uploader': that.uploader, 'id': id })
                console.log('New fileclass id: ', id)
                that.splits.push(id)
                console.log('Splits: ', that.splits)
                return
            })
          })).then(() => {
            that.updateUploadStatus('splits', that.splits)
            console.log('done splitting', that.id)
            resolve()
          })      
      } else {
        resolve()
      }
    })
  }

  //add all the files to file storage
  async addToStorage() {
    console.log('into storageAdd', this.id)
    let files = await this.listFiles()
    
      await Promise.all(
        files.map(async (file) => {

          var inFile = `${this.currentPath}/${file}`
          if (!this.reanalyze) {
            let file_sha256 = await hasha.fromFile(`${this.currentPath}/${file}`, {
              algorithm: 'sha256',
            })
            
            var outFile = await fileService.getDbFilePath(file_sha256)
            console.log('infile: ', inFile)
            console.log('outfile: ', outFile)
            await fileService.encryption(inFile, outFile, "encrypt")
          }
          await fsPromises.rm(inFile)
          return
        }))
    console.log('done adding to storage', this.id)
    this.updateUploadStatus('added_storage', true)
    this.remove()
  }

  //update status in the database
  async updateUploadStatus(property, value) {
    let updated = {}
    updated[property] = value
    uploadModel.findOneAndUpdate(
      { fileId: this.id },
      updated,
      { new: true, projection: { _id: false, __v: false } })
      .catch((err) => {
      console.log('cant update uploadStatus. ID: ', this.id, err)
    }).then((ret) => {
      this.eventEmitter.emit(this.id, ret)
      return 
    })
  }

  //rename files to their hash. Is done before sending to unzip and analysis
  async renameFiles(file = null) {
    if (file) {
      let file_sha256 = await hasha.fromFile(`${this.currentPath}/${file}`, {
        algorithm: 'sha256',
      })
      await fsPromises.rename(`${this.currentPath}/${file}`, `${this.currentPath}/${file_sha256}`)
    } else {
      console.log('listfiles:', this.id)
      let files = await this.listFiles()
      await Promise.all(
        files.map(async (file) => {
          let file_sha256 = await hasha.fromFile(`${this.currentPath}/${file}`, {
            algorithm: 'sha256',
          })
          await fsPromises.rename(`${this.currentPath}/${file}`, `${this.currentPath}/${file_sha256}`)
            .catch(err => {
            console.error('failed to rename: ', file)
          })
        }))
    } 
    return
  }

  //create the directory where the file object issa
  async mkTmpDir(path) {
    return fsPromises.mkdtemp(path).catch(err => {
      console.error(err)
    })
  }

  async main(folder, filenames) {
    let upload = new uploadModel({
      fileId: this.id
    })
    upload.save().catch(err => {
      console.error(err)
    })
    if (folder && filenames) {
      await this.addFiles(folder, filenames)
    }
    
    await this.split()
    await this.unzip()
    if (this.currentDir != process.env.unzippableFilePath) {
      if (!this.reanalyze) {
        await this.deleteExisting()
      }
      if ((await this.listFiles()).length) {
        await this.addDbEntry()
        await this.renameFiles()
        this.analyze()
      } else {
        this.remove()
      }
      
    }
  }
}

module.exports = fileClass
